/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <iostream>
#include <fstream>
#include <sys/socket.h>
#include "Config.h"
#include "AccessPointList.h"

using namespace std;

Config::Config()
{
    dirname = new string("no directory set");
    interface = NULL;
	dhcpCommand = new string ("pump -i ");

    /* Create a channel to the NET kernel. */
    int skfd;
    if((skfd = socket(AF_INET, SOCK_DGRAM,0) ) < 0)
    {
        cout << "socket creation in Config::Config()" << endl;
    }
	setSkfd(skfd);
}


void Config::setMain(Main* m)
{
    main = m;
}


Main* Config::getMain()
{
    return main;
}


void Config::setWindow(Window* w)
{
    win = w;
}


Window* Config::getWindow()
{
    return win;
}


void Config::setSkfd(int s)
{
    skfd = s;
}


int Config::getSkfd()
{
    return skfd;
}


void Config::setInterface(NetworkInterface* iface)
{
    if(iface != NULL)
    {
        cout << "config: setting interface to " << iface->getName() << endl;
    }
    interface = iface;
}


NetworkInterface* Config::getInterface()
{
    return interface;
}


NetworkInterface* Config::findInterface()
{
    NetworkInterface *ifn;
    ifn = new NetworkInterface();

    // Pull interface names out of /proc/net/wireless
    ifstream *in;
    in = new ifstream("/proc/net/wireless");

    char line[128];
    string *s;
    string *nss;
    while (in->getline(line,127))
    {
        s = new string(line);
        string::size_type colonpos;
        colonpos = s->find_first_of(':');
        // 4294967295 -> this is size_type's idea of zero?
        if(colonpos < 100000000 )
        {
            // We've got an interface
            string::size_type nospacepos;
            nospacepos = s->find_first_not_of(' ');
            // I dont know how to assign this to s, a string*, so i went with nss, a string
            nss = new string(s->substr(nospacepos, colonpos-2));
        	ifn->setName(nss);
	        return ifn;
        }
        delete s;
    }
    delete in;

	return NULL;

}


void Config::setAPs(AccessPointList* apl)
{
    aplist = apl;
}


AccessPointList* Config::getAPs()
{
    return aplist;
}


void Config::syncAPList(AccessPointList* apl)
{
    // mark all the APs as 'gone away'
    // the mark any APs in apl as 'here'

    // Compare the incoming list with the existing AP List
    cout << "Syncing old APList size " << aplist->size() << " addr:" << aplist << " with new AP list size " << apl->size() << " addr:" << apl << endl;
    AccessPointList::const_iterator newit;
    newit = apl->begin();
    for(;newit != apl->end(); ++newit)
    {
        AccessPointList::const_iterator oldit;
        oldit = aplist->begin();
        cout << "oldit aplist->begin() " << endl;
        bool addme = true;
        for(;oldit != aplist->end(); ++oldit)
        {
            if( (**oldit) == (**newit) )
            {
                // Already have this AP
                cout << "Already have AP bssid: " << (*newit)->getBssidString() << endl;
                addme = false;
            }
            else
            {
                cout << "New AP bssid: " << (*newit)->getBssidString() << endl;
            }
        }
        if(addme == true)
        {
            cout << "Adding AP Essid:" << (*newit)->getEssid() << " addr:" << (*newit) << " " << endl;
            //AccessPoint* ap0 = *newit;
            win->add_AP(*newit);
            aplist->push_back((*newit));
        }
    }
    cout << "SyncAPs finish.  aplist->size() " << aplist->size() << endl;
}


void Config::setPhantomAPs(bool pap)
{
    PhantomAPs = pap;
}


bool Config::getPhantomAPs()
{
    return PhantomAPs;
}


void Config::setDirname(string* dn)
{
    if(dn != 0)
    {
        if(dn->length() > 0)
        {
            dirname = dn;
            //cout << "Config::setDirname setting " << *dn << endl;
        }
        else
        {
            cout << "Config::setDirname given 0 length string" << endl;
        }
    }
    else
    {
        cout << "Config::setDirname given 0 pointer" << endl;
    }
}


string* Config::getDirname()
{
    //cout << "Config::getDirname HERE returning " << " addr:" << dirname << endl;
	//if(dirname != NULL) {
	//	cout << "value: " << *dirname << endl;
	//}
    return dirname;
}

void Config::setDhcpCommand(string* dc)
{
		dhcpCommand = dc;
}

string* Config::getDhcpCommand()
{
		return dhcpCommand;
}


/*
 * Copyright (c) 2002 Donald G. Park
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
