/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <string>
#include <iostream>
#include <sstream>
#include <arpa/inet.h>
#include <netinet/ip_icmp.h>
#include "iwlib.h"
#include "NetworkInterface.h"
#include "AccessPoint.h"
#include "Config.h"

// The constructor
//NetworkInterface::NetworkInterface() {
//}

// The destructor
//NetworkInterface::~NetworkInterface() {
//}

void NetworkInterface::setName(string* n)
{
    name = n;
}


string* NetworkInterface::getName()
{
    return name;
}


void NetworkInterface::Associate(AccessPoint* ap)
{
    // Set this interface to the parameters specified in the passed-in AccessPoint
    setEssid(ap->getEssid());
    setMode(ap->getMode());

	// Kick off a DHCP request
	ostringstream o;
	o << *(getConfig()->getDhcpCommand()) << *getName() << " & ";
	string* cmdline = new string(o.str());
	cout << "Launching \"" << *cmdline << "\"" << endl;
	system(cmdline->c_str());
	
	// pull the Gateway out of /etc/dhcpcd/dhcpcd-ifname.info
	
}


void NetworkInterface::setEssid(string* e)
{
    cout << "Setting Essid " << *e << " on " << *name << endl;
    //  IW_SET_EXT_ERR(skfd, ifname, SIOCSIWESSID, &wrq, "Set ESSID");
    struct iwreq wrq;
    wrq.u.essid.flags = 1;
    wrq.u.essid.pointer = (caddr_t) e->c_str();
    wrq.u.essid.length = strlen(e->c_str()) + 1;
    if(iw_set_ext(getConfig()->getSkfd(), (char*)getName()->c_str(), SIOCSIWESSID, &wrq) < 0)
    {
        cout << "Error setting essid" << endl;
    }

}


string* NetworkInterface::getEssid()
{
    //	return essid;
    /* Get ESSID */
    struct iwreq wrq;
    char str[64];
    wrq.u.essid.pointer = (caddr_t) str;
    wrq.u.essid.length = IW_ESSID_MAX_SIZE;
    wrq.u.essid.flags = 0;
    if(iw_get_ext(getConfig()->getSkfd(), (char*)name->c_str(), SIOCGIWESSID, &wrq) >= 0)
    {
        return new string(str);
    }

    return new string("unknown");
}


void NetworkInterface::setBssid(char* b)
{
}


char* NetworkInterface::getBssid()
{
    return "";
}


void NetworkInterface::setMode(int m)
{
    struct iwreq wrq;
	//IW_SET_EXT_ERR(skfd, ifname, SIOCSIWMODE, &wrq, "Set Mode");
	wrq.u.mode = m;
    if(iw_set_ext(getConfig()->getSkfd(), (char*)getName()->c_str(), SIOCSIWMODE, &wrq) < 0)
    {
        cout << "Error setting operation mode" << endl;
    }

}


int NetworkInterface::getMode()
{

  struct iwreq wrq;
  /* Get operation mode */
  if(iw_get_ext(getConfig()->getSkfd(), (char*)name->c_str(), SIOCGIWMODE, &wrq) >= 0)
  {
    return wrq.u.mode;
  }

    return -1;
}

string* NetworkInterface::getModeString()
{
	int mode = getMode();
	if(mode == -1)
	    return new string("unknown");
	if(mode == 1)
	    return new string("ad-hoc");
	if(mode == 2)
	    return new string("managed");
}

void NetworkInterface::setWEP(int w)
{
}


int NetworkInterface::getWEP()
{
    return 0;
}


void NetworkInterface::setFrequency(float f)
{
}


float NetworkInterface::getFrequency()
{
    return 0;
}


void NetworkInterface::ping(string* host)
{
    // ping the first default gateway that uses this interface
    int icmp_sock;                                /* socket file descriptor */
    struct sockaddr_in whereto;                   /* who to ping */
    struct hostent *hp;

    icmp_sock = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
    if(icmp_sock == -1)
    {
        fprintf(stdout,"ping socket creation error. not running as root?\n");
		return;
    }

    //source.sin_family = AF_INET;

    bzero((char *)&whereto, sizeof(whereto));
    whereto.sin_family = AF_INET;
    // we've got a hostname
    hp = gethostbyname(host->c_str());
    if (!hp)
    {
        fprintf(stderr, "ping: unknown host %s\n", host->c_str());
    }
    memcpy(&whereto.sin_addr, hp->h_addr, 4);
    whereto.sin_port = 0;
    fprintf(stdout, "pinging %s %s\n", host->c_str(), inet_ntoa(whereto.sin_addr));

    struct echorequest
    {
        icmphdr icmpheader;
        long   dwTime;
        char   cData[4];
    } erequest;

    strcpy(erequest.cData, "abc");
    erequest.icmpheader.type        = ICMP_ECHO;
    erequest.icmpheader.code        = 0;
    erequest.icmpheader.checksum    = 0;
    erequest.icmpheader.un.echo.id       = nPingId++;
    erequest.icmpheader.un.echo.sequence = nPingSeq++;

    erequest.icmpheader.checksum = ipchecksum((unsigned short*)&erequest,
        sizeof(erequest.icmpheader),0);

    struct sockaddr to;
    bzero((char *)&to, sizeof(sockaddr));
    to.sa_family = AF_INET;
    memcpy(to.sa_data,&whereto, 14);

    int retval;
    retval = sendto(icmp_sock,                    /* socket */
        &erequest,                                /* buffer */
        sizeof(echorequest),
        0,                                        /* flags */
        &to,                                      /* destination */
        sizeof(sockaddr));                        /* address length */

    if (retval == -1)
        fprintf(stdout, "ping send error");

}


Ipv4Address* NetworkInterface::getIpv4Address() {
    struct ifreq ifr;
    struct sockaddr_in *sin = (struct sockaddr_in *)&ifr.ifr_addr;
    int sockfd, i;

    bzero(&ifr, sizeof(ifr));

	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if(getName() != NULL) {
		cout << "getting IP for " << *getName() << endl;
	    strcpy(ifr.ifr_name, getName()->c_str());
	} else {
    	return NULL;
		cout << "cant get IP. no interface name. " <<  endl;
	}
    sin->sin_family = AF_INET;

    if(ioctl(sockfd, SIOCGIFADDR, &ifr) == 0)
    {
		unsigned int addr = sin->sin_addr.s_addr;
		return new Ipv4Address( addr & 0xff, 
					          	(addr & 0xff00) >> 8, 
								(addr & 0xff0000) >> 16, 
								(addr & 0xff000000) >> 24);
    }

	cout << "getting IP for " << *getName()  << " failed." << endl;
	return NULL;
}

unsigned short NetworkInterface::ipchecksum(unsigned short *addr, register int len, int csum)
{

    register int nleft = len;
    const u_short *w = addr;
    register u_short answer;
    register int sum = csum;

    /*
     *  Our algorithm is simple, using a 32 bit accumulator (sum),
     *  we add sequential 16 bit words to it, and at the end, fold
     *  back all the carry bits from the top 16 bits into the lower
     *  16 bits.
     */
    while (nleft > 1)
    {
        sum += *w++;
        nleft -= 2;
    }

    /* mop up an odd byte, if necessary */
    if (nleft == 1)
        sum += htons(*(u_char *)w << 8);

    /*
     * add back carry outs from top 16 bits to low 16 bits
     */
    sum = (sum >> 16) + (sum & 0xffff);           /* add hi 16 to low 16 */
    sum += (sum >> 16);                           /* add carry */
    answer = ~sum;                                /* truncate to 16 bits */
    return (answer);
}


/*
 * Copyright (c) 2002 Donald G. Park
 * The MT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
